package com.utopiaindustries.service;

import com.utopiaindustries.dao.ctp.*;
import com.utopiaindustries.model.ctp.*;
import org.springframework.stereotype.Service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class DashboardService {

    private final JobCardItemDAO jobCardItemDAO;
    private final ProcessDAO processDAO;
    private final BundleDAO bundleDAO;
    private final InventoryTransactionLegDAO inventoryTransactionLegDAO;
    private final InventoryTransactionDAO inventoryTransactionDAO;
    private final JobCardDAO jobCardDAO;
    private final CryptographyService cryptographyService;
    private final MasterBundleDAO masterBundleDAO;
    private final FinishedItemDAO finishedItemDAO;
    private final StitchingOfflineItemDAO stitchingOfflineItemDAO;
    private final InventoryAccountDAO inventoryAccountDAO;
    private final PackagingItemsDAO packagingItemsDAO;


    public DashboardService(JobCardItemDAO jobCardItemDAO, ProcessDAO processDAO, BundleDAO bundleDAO, InventoryTransactionLegDAO inventoryTransactionLegDAO, InventoryTransactionDAO inventoryTransactionDAO, JobCardDAO jobCardDAO, CryptographyService cryptographyService, MasterBundleDAO masterBundleDAO, FinishedItemDAO finishedItemDAO, StitchingOfflineItemDAO stitchingOfflineItemDAO, InventoryAccountDAO inventoryAccountDAO, PackagingItemsDAO packagingItemsDAO) {
        this.jobCardItemDAO = jobCardItemDAO;
        this.processDAO = processDAO;
        this.bundleDAO = bundleDAO;
        this.inventoryTransactionLegDAO = inventoryTransactionLegDAO;
        this.inventoryTransactionDAO = inventoryTransactionDAO;
        this.jobCardDAO = jobCardDAO;
        this.cryptographyService = cryptographyService;
        this.masterBundleDAO = masterBundleDAO;
        this.finishedItemDAO = finishedItemDAO;
        this.stitchingOfflineItemDAO = stitchingOfflineItemDAO;
        this.inventoryAccountDAO = inventoryAccountDAO;
        this.packagingItemsDAO = packagingItemsDAO;
    }

    public Map<String, Float> getPhasesProgressDayWise() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        String cuttingAccount = "CUTTING ACCOUNT " +1;
        String stitchingAccount = "STITCHING ACCOUNT " +1;
        String finishingAccount = "FINISHING ACCOUNT " +1;
        String packagingAccount = "A GRADE ACCOUNT " +1;

        String startDate1 = LocalDateTime.now().withHour(0).withMinute(0).withSecond(1).format(formatter);
        String endDate1 = LocalDateTime.now().withHour(23).withMinute(59).withSecond(59).format(formatter);


        HashMap<String, Float> progress = new HashMap<>();

        List<InventoryAccount> inventoryAccounts = inventoryAccountDAO.findAll();

        List<Integer> inventoryAccountIds = inventoryAccounts.stream()
                .filter(e -> cuttingAccount.equals(e.getTitle()) ||
                        stitchingAccount.equals(e.getTitle()) ||
                        finishingAccount.equals(e.getTitle()) ||
                        packagingAccount.equals(e.getTitle()))
                .map(e -> (int) e.getId())
                .collect(Collectors.toList());


        List<InventoryTransactionLeg> inventoryTransactionLeg = inventoryTransactionLegDAO.getTransactionByAndDatesAndAccountID(startDate1, endDate1, inventoryAccountIds);


        List<StitchingOfflineItem> stitchingOfflineItems =stitchingOfflineItemDAO.findByQCDoneDate(startDate1, endDate1);

        List<FinishedItem> finishedItems = finishedItemDAO.findByDate(startDate1, endDate1);
        List<PackagingItems> packagingItems = packagingItemsDAO.findByDate(startDate1, endDate1);

        float stitchingDefectedPiece =  stitchingOfflineItems.stream()
                .filter(e -> "REJECT".equals(e.getQaStatus()))
                .count();

        float finishedItemDefected =  finishedItems.stream()
                .filter(e -> "ALTER".equals(e.getQaStatus()))
                .count();

        float finishedItem =finishedItems.stream()
                .filter(e -> "APPROVED".equals(e.getQaStatus()))
                .count();

        float finishedItemAGrade = packagingItems.size();

        float finishedItemBGrade = finishedItems.stream()
                .filter(e -> "B GRADE".equals(e.getQaStatus()))
                .count();

        float finishedItemCGrade = finishedItems.stream()
                .filter(e -> "C GRADE".equals(e.getQaStatus()))
                .count();

        float finishDefectedPiece = finishedItemDefected + finishedItemBGrade + finishedItemCGrade;

        progress.put("Stitching", (float) stitchingOfflineItems.size());
        progress.put("StitchingDefectPiece", stitchingDefectedPiece);
        progress.put("StitchingDefectPieceRatio", stitchingDefectedPiece / stitchingOfflineItems.size());

        progress.put("finishing", (float) finishedItems.size());
        progress.put("finishingDefectPiece", finishDefectedPiece);
        progress.put("finishingDefectPieceRatio", finishDefectedPiece / finishedItems.size());


        progress.put("QA", (float) finishedItems.size());
        progress.put("Finishing", finishedItem);
        progress.put("A Grade", finishedItemAGrade);
        progress.put("B Grade", finishedItemBGrade);
        progress.put("C Grade", finishedItemCGrade);

        return progress;
    }

    public Map<String, String> getLineDetails() {
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

        String startDate1 = LocalDateTime.now().withHour(0).withMinute(0).withSecond(1).format(formatter);
        String endDate1 = LocalDateTime.now().withHour(23).withMinute(59).withSecond(59).format(formatter);
        String startDateWithHour =  LocalDateTime.now().minusHours(1).format(formatter);
        String endDateWithHour =LocalDateTime.now().format(formatter);


        HashMap<String, String> details = new HashMap<>();
        List<PackagingItems> packagingItems = packagingItemsDAO.findByDate(startDate1, endDate1);
        List<PackagingItems> packagingItemsHourly = packagingItemsDAO.findByDate(startDateWithHour, endDateWithHour);

        List<JobCard> jobCards = jobCardDAO.findAll();
        int target = 1000;
        int achieved = packagingItems.size();


        details.put("Total Target", 1000 + " pc");
        details.put("Total Achieved", achieved + " pc");
        details.put("Hourly Target", 1000/24 + " pc");
        details.put("Hourly Achieved", packagingItemsHourly.size() + " pc");
        details.put("Efficiency",(float)achieved / target * 100 + " %" );
        details.put("Total Induction", String.valueOf(jobCards.size()));
        details.put("Total Machine", String.valueOf(30));
        details.put("Total Worker", String.valueOf(30));
        return details;
    }
}
